import datetime

from zope.interface import implementer
from zope.interface import implements
from zope.component import adapter

from zope.publisher.interfaces import IRequest

from App.config import getConfiguration
from collective.beaker.interfaces import ISession

class BeakerConfigLayer(object):
    """This layer sets up beaker configuration as if it had been present
    in zope.conf. The configuration data is held in self.zconfigData
    """
    
    # Simulated ZConfig data
    zconfigData = {
        'cache.type':           'memory',
        'cache.regions':        'short, long',
        'cache.short.expire':   '3',
        'cache.long.expire':    '10',
        'session.type':         'memory',
        'session.key':          'beaker.session',
        'session.auto':         'off',
    }
    
    @classmethod
    def setUp(self):
        cfg = getConfiguration()
        cfg.product_config = {'beaker': self.zconfigData}

class TestSession(dict):
    """Fake session object that can be used for unit testing
    """
    implements(ISession)
    
    id = 'test-session'
    last_accessed = None
    
    # testing variables
    _accessed = False
    _saved = False
    _deleted = False
    _invalidated = False
    
    def accessed(self):
        return self._accessed
    
    def save(self):
        self._saved = True
    
    def delete(self):
        self._deleted = True
    
    def invalidate(self):
        self._invalidated = True
    
    # make accessed work at least for common cases
    
    def __getitem__(self, key):
        self._accessed = True
        self.last_accessed = datetime.datetime.now()
        return super(TestSession, self).__getitem__(key)
    
    def __contains__(self, key):
        self._accessed = True
        self.last_accessed = datetime.datetime.now()
        return super(TestSession, self).__contains__(key)
    
    def get(self, key, default=None):
        self._accessed = True
        self.last_accessed = datetime.datetime.now()
        return super(TestSession, self).get(key, default)
    
    def __setitem__(self, key, value):
        self._accessed = True
        self.last_accessed = datetime.datetime.now()
        super(TestSession, self).__setitem__(key, value)
    
    def setdefault(self, key, value):
        self._accessed = True
        self.last_accessed = datetime.datetime.now()
        return super(TestSession, self).setdefault(key, value)
    
    def __delitem__(self, key):
        self._accessed = True
        self.last_accessed = datetime.datetime.now()
        super(TestSession, self).__delitem__(key)

@implementer(ISession)
@adapter(IRequest)
def testingSession(request):
    return request._environ.setdefault('collective.beaker.testing.session', TestSession())



from zope.interface import implementer
from zope.component import adapter, queryUtility

from zope.publisher.interfaces.http import IHTTPRequest

from collective.beaker.interfaces import ISession, ISessionConfig, ENVIRON_KEY
from ZPublisher.interfaces import IPubStart, IPubSuccess, IPubFailure

from beaker.session import SessionObject

@implementer(ISession)
@adapter(IHTTPRequest)
def ZopeSession(request):
    """Adapter factory from a Zope request to a beaker session
    """
    return request.environ.get(ENVIRON_KEY, None)

# Helper functions

def initializeSession(request, environ_key='beaker.session'):
    """Create a new session and store it in the request.
    """
    options = queryUtility(ISessionConfig)
    if options is not None:
        session = SessionObject(request.environ, **options)
        request.environ[ENVIRON_KEY] = session

def closeSession(request):
    """Close the session, and, if necessary, set any required cookies
    """
    session = ISession(request, None)
    if session is not None:
        if session.accessed():
            session.persist()
            sessionInstructions = session.request
            if sessionInstructions.get('set_cookie', False):
                if sessionInstructions['cookie_out']:
                    cookieObj = session.cookie[session.key]
                    
                    key = cookieObj.key
                    value = session.cookie.value_encode(cookieObj.value)[1]
                    
                    args = dict([(k,v) for k,v in cookieObj.items() if v])
                    args.setdefault('path', session._path)
                    
                    request.response.setCookie(key, value, **args)

# Event handlers

@adapter(IPubStart)
def configureSessionOnStart(event):
    initializeSession(event.request)

@adapter(IPubSuccess)
def persistSessionOnSuccess(event):
    closeSession(event.request)

@adapter(IPubFailure)
def persistSessionOnFailure(event):
    if not event.retry:
        closeSession(event.request)



from setuptools import setup, find_packages
import os

version = '1.0b2'

setup(name='collective.beaker',
      version=version,
      description="Beaker integration for Zope and Plone",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='beaker zope plone caching sessions',
      author='Martin Aspeli',
      author_email='optilude@gmail.com',
      url='http://pypi.python.org/pypi/collective.beaker',
      license='BSD',
      packages=find_packages('src', exclude=['ez_setup']),
      package_dir = {'':'src'},
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'beaker',
          'zope.interface',
          'zope.component',
          'Zope2',
          'zope.publisher',
      ],
      extras_require={
        'Zope2.10': ['ZPublisherEventsBackport'],
        'tests': ['Products.PloneTestCase', 'collective.testcaselayer']
      },
      entry_points="""
      """,
      )
